local AceGUI = LibStub("AceGUI-3.0")

local loot_id, loot_link
local classes
local bids
local raidinfo
local selected_player, selected_bid

function DEDKP_BidManager_SetItem(item_link)
	if DERR_INBID then return end
	
	if DERR.BidManager == nil then
		DEDKP_BidManager_Show()
	else
		DEDKP_BidManager_Clear_All()
	end

	DERR.BidManager.LootContainer:ReleaseChildren()
	DERR.BidManager.LootIcon = AceGUI:Create("Icon")
	DERR.BidManager.LootIcon:SetImageSize(50,50)
	DERR.BidManager.LootIcon:SetCallback("OnEnter",DEDKP_BidManager_LootIcon_OnEnter)
	DERR.BidManager.LootIcon:SetCallback("OnLeave",DEDKP_BidManager_LootIcon_OnLeave)
	DERR.BidManager.LootContainer:AddChild(DERR.BidManager.LootIcon)

	DERR.BidManager.LootText = AceGUI:Create("Label")
	DERR.BidManager.LootText:SetWidth(175)
	DERR.BidManager.LootText:SetJustifyH("CENTER")
	DERR.BidManager.LootContainer:AddChild(DERR.BidManager.LootText)
	
	loot_id = tonumber(select(3, strfind(item_link, "item:(%d+)")))



	local item = Item:CreateFromItemID(loot_id)
	item:ContinueOnItemLoad(function()
		local itemName,itemLink,itemRarity,_,_,_,_,_,_,itemIcon = GetItemInfo(loot_id)
		loot_link = itemLink
		local r, g, b, hex = GetItemQualityColor(itemRarity)
		DERR.BidManager.LootIcon:SetImage(itemIcon)
		DERR.BidManager.LootText:SetText("|c"..hex..itemName.."|r")
		DERR.BidManager.StartButton:SetDisabled(false)
		DERR.BidManager.ShardButton:SetDisabled(false)
		DERR.BidManager.BankButton:SetDisabled(false)
		DERR.BidManager.MinBidSlider:SetValue(40)
		local lp = DEDKP_LOOT_PRIORITY[loot_id]
		if lp ~= nil then
			local mindkp = tonumber(lp.dkp)
			if mindkp ~= nil then
				DERR.BidManager.MinBidSlider:SetValue(mindkp)
			end
		end

	end)


end

function DEDKP_BidManager_Clear_All()
	DERR.BidManager.BidScroll:ReleaseChildren()
	DERR.BidManager.ClassGroup:ReleaseChildren()
	DERR.BidManager.LootContainer:ReleaseChildren()
	DERR.BidManager.AwardButton:SetDisabled(true)
	DERR.BidManager.StartButton:SetDisabled(true)
	DERR.BidManager.ShardButton:SetDisabled(true)
	DERR.BidManager.BankButton:SetDisabled(true)

	classes = {}
	for class, classInfo in pairs(dedkp_class_info) do
		if ((class == "SHAMAN" and faction == "Horde") or (class=="PALADIN" and faction == "Alliance") or (class~="PALADIN" and class~="SHAMAN")) then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel(DraconisErus_Class_Colorize(class,classInfo.localName))
			check.class = class
			check:SetWidth(80)
			check:SetCallback("OnValueChanged", DEDKP_BidManager_Class_OnValueChanged)
			DERR.BidManager.ClassGroup:AddChild(check)
		end
	end	
end

function DEDKP_BidManager_Init()
	classes = {}
	bids = {}
	raiders = {}
	selected_player = ""
	selected_bid = 0
end

function DEDKP_BidManager_Show()
	DEDKP_BidManager_Init()

	DERR.BidManager = AceGUI:Create("Frame")
	DERR.BidManager:SetTitle("Draconis Erus Bid Manager")
	DERR.BidManager:SetLayout("Flow")
	DERR.BidManager:SetWidth(410)
	DERR.BidManager:SetHeight(470)
	DERR.BidManager:EnableResize(false)
	DERR.BidManager:SetCallback("OnClose", DEDKP_BidManager_OnClose)

	DERR.BidManager.HeaderLabel = AceGUI:Create("Label")
	DERR.BidManager.HeaderLabel:SetText("Ctrl + Alt + Right Click to add item.")
	DERR.BidManager.HeaderLabel:SetFullWidth(true)
	DERR.BidManager.HeaderLabel:SetJustifyH("CENTER")
	DERR.BidManager:AddChild(DERR.BidManager.HeaderLabel)


		--Bid Scroll
	DERR.BidManager.BidContainer = AceGUI:Create("InlineGroup")
	DERR.BidManager.BidContainer:SetLayout("Fill")
	DERR.BidManager.BidContainer:SetWidth(185)
	DERR.BidManager.BidContainer:SetHeight(115)
	DERR.BidManager.BidContainer:SetTitle("Bid List:")
	DERR.BidManager:AddChild(DERR.BidManager.BidContainer)

	DERR.BidManager.BidScroll = AceGUI:Create("ScrollFrame")
	DERR.BidManager.BidScroll:SetLayout("Flow")
	DERR.BidManager.BidContainer:AddChild(DERR.BidManager.BidScroll)

	DERR.BidManager.LootContainer = AceGUI:Create("InlineGroup")
	DERR.BidManager.LootContainer:SetLayout("DERR_Layout")
	DERR.BidManager.LootContainer:SetWidth(185)
	DERR.BidManager.LootContainer:SetHeight(115)
	DERR.BidManager.LootContainer:SetTitle("Loot:")
	DERR.BidManager:AddChild(DERR.BidManager.LootContainer)

	DERR.BidManager.Winner = AceGUI:Create("Label")
	DERR.BidManager.Winner:SetWidth(185)
	DERR.BidManager:AddChild(DERR.BidManager.Winner)


	DERR.BidManager.BidTimerBar = AceGUI:Create("TimerBar")
	DERR.BidManager.BidTimerBar:SetWidth(390)
	DERR.BidManager.BidTimerBar:SetHeight(10)
	DERR.BidManager.BidTimerBar:SetCallback("OnEnd",DEDKP_BidManager_BidTimerBar_OnEnd)
	DERR.BidManager:AddChild(DERR.BidManager.BidTimerBar)

	DERR.BidManager.AwardContainer = AceGUI:Create("SimpleGroup")
	DERR.BidManager.AwardContainer:SetLayout("DERR_BidManager_Layout")
	DERR.BidManager.AwardContainer:SetWidth(185)
	DERR.BidManager.AwardContainer:SetHeight(50)
	DERR.BidManager:AddChild(DERR.BidManager.AwardContainer)

	DERR.BidManager.AwardButton = AceGUI:Create("Button")
	DERR.BidManager.AwardButton:SetWidth(120)
	DERR.BidManager.AwardButton:SetHeight(40)
	DERR.BidManager.AwardButton:SetText("Award Loot")
	DERR.BidManager.AwardButton:SetDisabled(true)
	DERR.BidManager.AwardButton:SetCallback("OnClick",DEDKP_BidManager_AwardButton_OnClick)
	DERR.BidManager.AwardContainer:AddChild(DERR.BidManager.AwardButton)

	DERR.BidManager.BiddingContainer = AceGUI:Create("SimpleGroup")
	DERR.BidManager.BiddingContainer:SetLayout("DERR_BidManager_Layout")
	DERR.BidManager.BiddingContainer:SetWidth(185)
	DERR.BidManager.BiddingContainer:SetHeight(50)
	DERR.BidManager:AddChild(DERR.BidManager.BiddingContainer)

	DERR.BidManager.StartButton = AceGUI:Create("Button")
	DERR.BidManager.StartButton:SetWidth(120)
	DERR.BidManager.StartButton:SetHeight(40)
	DERR.BidManager.StartButton:SetText("Start Bidding")
	DERR.BidManager.StartButton:SetDisabled(true)
	DERR.BidManager.StartButton:SetCallback("OnClick",DEDKP_BidManager_StartButton_OnClick)
	DERR.BidManager.BiddingContainer:AddChild(DERR.BidManager.StartButton)

	DERR.BidManager.ClassContainer = AceGUI:Create("InlineGroup")
	DERR.BidManager.ClassContainer:SetLayout("Fill")
	DERR.BidManager.ClassContainer:SetWidth(185)
	DERR.BidManager.ClassContainer:SetHeight(140)
	DERR.BidManager.ClassContainer:SetTitle("Class:")
	DERR.BidManager:AddChild(DERR.BidManager.ClassContainer)

	DERR.BidManager.ClassGroup = AceGUI:Create("SimpleGroup")
	DERR.BidManager.ClassGroup:SetLayout("Flow")
	DERR.BidManager.ClassContainer:AddChild(DERR.BidManager.ClassGroup)

	faction = UnitFactionGroup("player")
	for class, classInfo in pairs(dedkp_class_info) do
		if ((class == "SHAMAN" and faction == "Horde") or (class=="PALADIN" and faction == "Alliance") or (class~="PALADIN" and class~="SHAMAN")) then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel(DraconisErus_Class_Colorize(class,classInfo.localName))
			check.class = class
			check:SetWidth(80)
			check:SetCallback("OnValueChanged", DEDKP_BidManager_Class_OnValueChanged)
			DERR.BidManager.ClassGroup:AddChild(check)
		end
	end	

	DERR.BidManager.Settings = AceGUI:Create("InlineGroup")
	DERR.BidManager.Settings:SetLayout("Fill")
	DERR.BidManager.Settings:SetWidth(185)
	DERR.BidManager.Settings:SetHeight(140)
	DERR.BidManager.Settings:SetTitle("Settings:")
	DERR.BidManager:AddChild(DERR.BidManager.Settings)

	DERR.BidManager.SliderGroup = AceGUI:Create("SimpleGroup")
	DERR.BidManager.SliderGroup:SetLayout("Flow")
	DERR.BidManager.Settings:AddChild(DERR.BidManager.SliderGroup)

	DERR.BidManager.MinBidSlider = AceGUI:Create("Slider")
	DERR.BidManager.MinBidSlider:SetSliderValues(0,300,5)
	DERR.BidManager.MinBidSlider:SetValue(40)
	DERR.BidManager.MinBidSlider:SetLabel("Minimum Bid")
	DERR.BidManager.SliderGroup:AddChild(DERR.BidManager.MinBidSlider)

	DERR.BidManager.BidTimerSlider = AceGUI:Create("Slider")
	DERR.BidManager.BidTimerSlider:SetSliderValues(15,60,15)
	DERR.BidManager.BidTimerSlider:SetValue(30)
	DERR.BidManager.BidTimerSlider:SetLabel("Bid Timer")
	DERR.BidManager.SliderGroup:AddChild(DERR.BidManager.BidTimerSlider)

	DERR.BidManager.ShardContainer = AceGUI:Create("SimpleGroup")
	DERR.BidManager.ShardContainer:SetLayout("DERR_BidManager_Layout")
	DERR.BidManager.ShardContainer:SetWidth(185)
	DERR.BidManager.ShardContainer:SetHeight(50)
	DERR.BidManager:AddChild(DERR.BidManager.ShardContainer)

	DERR.BidManager.ShardButton = AceGUI:Create("Button")
	DERR.BidManager.ShardButton:SetWidth(120)
	DERR.BidManager.ShardButton:SetHeight(40)
	DERR.BidManager.ShardButton:SetText("Shard Loot")
	DERR.BidManager.ShardButton:SetDisabled(true)
	DERR.BidManager.ShardButton:SetCallback("OnClick",DEDKP_BidManager_ShardButton_OnClick)
	DERR.BidManager.ShardContainer:AddChild(DERR.BidManager.ShardButton)

	DERR.BidManager.BankContainer = AceGUI:Create("SimpleGroup")
	DERR.BidManager.BankContainer:SetLayout("DERR_BidManager_Layout")
	DERR.BidManager.BankContainer:SetWidth(185)
	DERR.BidManager.BankContainer:SetHeight(50)
	DERR.BidManager:AddChild(DERR.BidManager.BankContainer)

	DERR.BidManager.BankButton = AceGUI:Create("Button")
	DERR.BidManager.BankButton:SetWidth(120)
	DERR.BidManager.BankButton:SetHeight(40)
	DERR.BidManager.BankButton:SetText("Bank Loot")
	DERR.BidManager.BankButton:SetDisabled(true)
	DERR.BidManager.BankButton:SetCallback("OnClick",DEDKP_BidManager_BankButton_OnClick)
	DERR.BidManager.BankContainer:AddChild(DERR.BidManager.BankButton)

	DERR.BidManager.BidTimerBar:Hide()

	if next(DEDKPPoints[DERR_REALM][DERR_GUILD]) == nil then
		DEDKPPoints[DERR_REALM][DERR_GUILD] = DraconisErus_Guild_GetEmptyPoints()
	end

	raidinfo = DraconisErus_Raid_GetRaidInfo()

	DEDKP_BidManager_UpdateBids()

	DraconisErus_SetWindowPosition("BidManager", DERR.BidManager)

end

function DEDKP_BidManager_LootIcon_OnEnter(widget)
	GameTooltip:SetOwner(widget.frame)
	GameTooltip:SetPoint("TOPLEFT")
	GameTooltip:SetHyperlink(loot_link)
	GameTooltip:Show()
end

function DEDKP_BidManager_LootIcon_OnLeave()
	GameTooltip:Hide()
end

function DEDKP_BidManager_BidTimerBar_OnEnd()
	DEDKP_BidManager_EndTimer()
end

function DEDKP_BidManager_Class_OnValueChanged(widget, event, value)
	local className = widget.class
	if value then
		table.insert(classes,className)
	else
		local index = DraconisErus_TableSearch(classes,className)
		if index > 0 then
			table.remove(classes,index)
		end
	end	
end

function DEDKP_BidManager_StartButton_OnClick()
	if not DERR_INBID then
		raidinfo = DraconisErus_Raid_GetRaidInfo()

		DERR_INBID = true
		DERR.BidManager.AwardButton:SetDisabled(true)
		DERR.BidManager.ShardButton:SetDisabled(true)
		DERR.BidManager.BankButton:SetDisabled(true)
		DERR.BidManager.MinBidSlider:SetDisabled(true)
		DERR.BidManager.BidTimerSlider:SetDisabled(true)

		bids = {}
		DEDKP_BidManager_UpdateBids()

		local min_bid = DERR.BidManager.MinBidSlider:GetValue()
		local bid_timer = DERR.BidManager.BidTimerSlider:GetValue()
		local commdata = {}
		table.insert(commdata,loot_id)
		table.insert(commdata,min_bid)
		table.insert(commdata,bid_timer)
		if next(classes) == nil then
			table.insert(commdata,"")
		else
			table.insert(commdata,classes)
		end
		DERR_BIDDERS = {}
		for k,v in pairs(raidinfo) do
			if k ~= DERR_PLAYER and k ~= "raidleader" and k~="officers" and k~="nonofficers" then
				DraconisErus_Debug("name:" .. k)
				if next(classes) == nil then
					table.insert(DERR_BIDDERS,k)
				else
					if DraconisErus_TableSearch(classes,raidinfo[k].class) > 0 then
						table.insert(DERR_BIDDERS,k)
					end
				end
			end
		end

		table.insert(commdata,DraconisErus_Raid_GetPoints())
		DraconisErus_Comm_Send("DERRSTARTLOOT", commdata, "RAID")
		DERR.BidManager.StartButton:SetText("Stop Bidding")

		DERR.BidManager.BidTimerBar:SetMaxValue(bid_timer+5)
		DERR.BidManager.BidTimerBar:Start()

		local itemName,itemLink,itemRarity,_,_,_,_,_,_,itemIcon = GetItemInfo(loot_id)
		local tablelist = DraconisErus_TableToString(classes)
		if strlen(tablelist) > 0 then tablelist = " ("..tablelist..")" end
		SendChatMessage("Bidding has started for "..itemLink..tablelist..".","RAID_WARNING")
	else
		DEDKP_BidManager_EndTimer()
	end
end

function DEDKP_BidManager_EndTimer()
	DERR.BidManager.BidTimerBar:Stop()
	DraconisErus_Comm_Send("DERRENDLOOT", "nil", "RAID")
	DERR.BidManager.StartButton:SetText("Start Bidding")
	if next(bids) == nil then
		DERR.BidManager.ShardButton:SetDisabled(false)
		DERR.BidManager.BankButton:SetDisabled(false)
	end
	DERR.BidManager.MinBidSlider:SetDisabled(false)
	DERR.BidManager.BidTimerSlider:SetDisabled(false)
	DERR_INBID = false
	local itemName,itemLink,itemRarity,_,_,_,_,_,_,itemIcon = GetItemInfo(loot_id)
	SendChatMessage("Bidding has ended for "..itemLink..".","RAID_WARNING")

end

function DEDKP_BidManager_AddBidEntry(player, class, bid, points)
	local g = AceGUI:Create("SimpleGroup")
	g:SetLayout("Flow")

	local il = AceGUI:Create("InteractiveLabel")
	il:SetText(player)
	il:SetHighlight("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	il:SetWidth(80)
	il.player = player
	il:SetColor(dedkp_class_info[class].r,dedkp_class_info[class].g,dedkp_class_info[class].b)
	il:SetCallback("OnClick",DEDKP_BidManager_BidEntry_OnClick)
	g:AddChild(il)

	local l = AceGUI:Create("Label")
	il.bid = bid
	l:SetText(bid.."/"..points)
	l:SetWidth(60)
	l:SetColor(dedkp_class_info[class].r,dedkp_class_info[class].g,dedkp_class_info[class].b)
	l:SetJustifyH("RIGHT")
	g:AddChild(l)

	DERR.BidManager.BidScroll:AddChild(g)

end

function DEDKP_BidManager_UpdateBids()
	DERR.BidManager.BidScroll:ReleaseChildren()
	DEDKP_BidManager_SortBidTable()
	for index, value in pairs(bids) do
		local player = bids[index]["player"]
		local bid = bids[index]["bid"]
		local class = raidinfo[player].class
		local points = raidinfo[player].points
		DEDKP_BidManager_AddBidEntry(player, class, bid, points)
	end
end

function DEDKP_BidManager_BidEntry_OnClick(button)
	DERR.BidManager.AwardButton.player = button.player
	DERR.BidManager.AwardButton.bid = button.bid
	DERR.BidManager.AwardButton:SetText("Award "..button.player)
	DERR.BidManager.AwardButton:SetDisabled(false)
end

function DEDKP_BidManager_AddBid(player,bid)
	if not DERR_INBID then return end
	if bid < 0 then return end
	DraconisErus_Comm_Send("DERRBIDCONFIRM", bid, "WHISPER", player)

	local newBid = true
	for index, value in pairs(bids) do
		if bids[index]["player"] == player then
			bids[index]["bid"] = bid
			newBid = false
			break
		end
	end
	if newBid then
		table.insert(bids, { ["player"] = player, ["bid"] = bid } )
	end
	DEDKP_BidManager_UpdateBids()
end

function DEDKP_BidManager_AwardButton_OnClick(button)
	local winning_bid = button.bid
	if winning_bid < DERR.BidManager.MinBidSlider:GetValue() then winning_bid = DERR.BidManager.MinBidSlider:GetValue() end
	local players = DraconisErus_Raid_GetAllPlayers(false)
	local lootEntry = {
		["LootID"] = DraconisErus_GenerateID(false),
		["RaidID"] = DEDKPSettings["RaidID"],
		["ItemID"] = loot_id,
		["Action"] = "AWARD",
		["Player"] = button.player,
		["Bid"] = button.bid,
		["Amount"] = winning_bid,
		["Bids"] = bids,
		["Players"] = players
	}

	DraconisErus_RaidEvent_Loot(lootEntry)
	DEDKP_BidManager_OnClose(DERR.BidManager)
end

function DEDKP_BidManager_OnClose(widget)
	if DERR_INBID then DEDKP_BidManager_EndTimer() end
	if DERR.BidManager ~= nil then
		DraconisErus_SaveWindowPosition("BidManager", widget)
		widget:ReleaseChildren()
		widget:Hide()
		DERR.BidManager = nil
	end
end

function DEDKP_BidManager_ShardButton_OnClick()
	StaticPopupDialogs["DERR_SHARDPROMPT"] = {
		text = "Are you sure you wish to SHARD this loot?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
		DEDKP_BidManager_Shard()
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3,
	}
	StaticPopup_Show ("DERR_SHARDPROMPT")
	
end

function DEDKP_BidManager_Shard()
	local players = DraconisErus_Raid_GetAllPlayers(false)
	local bids = {}
	local lootEntry = {
		["LootID"] = DraconisErus_GenerateID(false),
		["RaidID"] = DEDKPSettings["RaidID"],
		["ItemID"] = loot_id,
		["Action"] = "SHARD",
		["Player"] = DERR_PLAYER,
		["Bid"] = 0,
		["Amount"] = 0,
		["Bids"] = bids,
		["Players"] = players
	}

	DraconisErus_RaidEvent_Loot(lootEntry)
	DEDKP_BidManager_OnClose(DERR.BidManager)
end

function DEDKP_BidManager_BankButton_OnClick()
	StaticPopupDialogs["DERR_BANKPROMPT"] = {
		text = "Are you sure you wish to BANK this loot?",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
		DEDKP_BidManager_Bank()
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3,
	}
	StaticPopup_Show ("DERR_BANKPROMPT")
end

function DEDKP_BidManager_Bank()
	local players = DraconisErus_Raid_GetAllPlayers(false)
	local bids = {}
	local lootEntry = {
		["LootID"] = DraconisErus_GenerateID(false),
		["RaidID"] = DEDKPSettings["RaidID"],
		["ItemID"] = loot_id,
		["Action"] = "BANK",
		["Player"] = DERR_PLAYER,
		["Bid"] = 0,
		["Amount"] = 0,
		["Bids"] = bids,
		["Players"] = players
	}

	DraconisErus_RaidEvent_Loot(lootEntry)
	DEDKP_BidManager_OnClose(DERR.BidManager)
end

function DEDKP_BidManager_SortBidTable()
	table.sort(bids, function(a, b)
	    	return a["bid"] > b["bid"]
  	end)
end

AceGUI:RegisterLayout("DERR_Layout",
	function(content, children)
		local top = 0
		for i = 1, #children do
			local child = children[i]
			child.frame:SetPoint("TOP",content,"TOP",0,top)
			child.frame:Show()
			top = top - child.frame:GetHeight() - 5
		end
	end)

AceGUI:RegisterLayout("DERR_BidManager_Layout",
	function(content, children)
		local top = 0
		for i = 1, #children do
			local child = children[i]
			child.frame:SetPoint("CENTER",content,"CENTER",0,top)
			child.frame:Show()
			top = top - child.frame:GetHeight()
		end
	end)

