local AceGUI = LibStub("AceGUI-3.0")

local loot_id, loot_link, tool_tip, points

function DEDKP_BidWindow_SetItem(item_id)
	loot_id = tonumber(item_id)

	local item = Item:CreateFromItemID(loot_id)
	item:ContinueOnItemLoad(function()
		local itemName,itemLink,itemRarity,_,_,_,_,_,_,itemIcon = GetItemInfo(loot_id)
		loot_link = itemLink
		local r, g, b, hex = GetItemQualityColor(itemRarity)
		DERR.BidWindow.LootIcon:SetImage(itemIcon)
		DERR.BidWindow.LootText:SetText("|c"..hex..itemName.."|r")
	end)

end

function DEDKP_BidWindow_Show(item_id, min_bid, timer)
	points = DEDKPPoints[DERR_REALM][DERR_GUILD][DERR_PLAYER]
	DERR.BidWindow = AceGUI:Create("Frame")
	DERR.BidWindow:SetTitle("Draconis Erus Bidding")
	DERR.BidWindow:SetLayout("Flow")
	DERR.BidWindow:SetWidth(410)
	DERR.BidWindow:SetHeight(260)
	DERR.BidWindow:EnableResize(false)
	DERR.BidWindow:SetCallback("OnClose", DEDKP_BidWindow_OnClose)

	DERR.BidWindow.LootContainer = AceGUI:Create("InlineGroup")
	DERR.BidWindow.LootContainer:SetLayout("DERR_Layout")
	DERR.BidWindow.LootContainer:SetWidth(185)
	DERR.BidWindow.LootContainer:SetHeight(165)
	DERR.BidWindow.LootContainer:SetTitle("Loot:")
	DERR.BidWindow:AddChild(DERR.BidWindow.LootContainer)

	DERR.BidWindow.LootIcon = AceGUI:Create("Icon")
	DERR.BidWindow.LootIcon:SetImageSize(50,50)
	DERR.BidWindow.LootIcon:SetCallback("OnEnter",DEDKP_BidWindow_LootIcon_OnEnter)
	DERR.BidWindow.LootIcon:SetCallback("OnLeave",DEDKP_BidWindow_LootIcon_OnLeave)
	DERR.BidWindow.LootContainer:AddChild(DERR.BidWindow.LootIcon)

	DERR.BidWindow.LootText = AceGUI:Create("Label")
	DERR.BidWindow.LootText:SetWidth(175)
	DERR.BidWindow.LootText:SetJustifyH("CENTER")
	DERR.BidWindow.LootContainer:AddChild(DERR.BidWindow.LootText)

	DERR.BidWindow.BidContainer = AceGUI:Create("InlineGroup")
	DERR.BidWindow.BidContainer:SetLayout("DERR_Layout")
	DERR.BidWindow.BidContainer:SetWidth(185)
	DERR.BidWindow.BidContainer:SetHeight(165)
	DERR.BidWindow.BidContainer:SetTitle("Bid:")
	DERR.BidWindow:AddChild(DERR.BidWindow.BidContainer)

	DERR.BidWindow.MinimumBid = AceGUI:Create("Label")
	DERR.BidWindow.MinimumBid:SetWidth(120)
	DERR.BidWindow.MinimumBid:SetText("Minimum Bid: "..min_bid)
	DERR.BidWindow.BidContainer:AddChild(DERR.BidWindow.MinimumBid)

	DERR.BidWindow.Points = AceGUI:Create("Label")
	DERR.BidWindow.Points:SetWidth(120)
	DERR.BidWindow.Points:SetText("Current Points: "..points)
	DERR.BidWindow.BidContainer:AddChild(DERR.BidWindow.Points)

	DERR.BidWindow.BidAmount = AceGUI:Create("EditBox")
	DERR.BidWindow.BidAmount:SetWidth(50)
	DERR.BidWindow.BidAmount:SetLabel("Your bid:")
	DERR.BidWindow.BidAmount:DisableButton(true)
	DERR.BidWindow.BidContainer:AddChild(DERR.BidWindow.BidAmount)

	DERR.BidWindow.BidButton = AceGUI:Create("Button")
	DERR.BidWindow.BidButton:SetWidth(120)
	DERR.BidWindow.BidButton:SetHeight(40)
	DERR.BidWindow.BidButton:SetText("Submit Bid")
	DERR.BidWindow.BidButton:SetCallback("OnClick",DEDKP_BidWindow_BidButton_OnClick)
	DERR.BidWindow.BidContainer:AddChild(DERR.BidWindow.BidButton)


	DERR.BidWindow.BidTimerBar = AceGUI:Create("TimerBar")
	DERR.BidWindow.BidTimerBar:SetWidth(390)
	DERR.BidWindow.BidTimerBar:SetHeight(10)
	DERR.BidWindow.BidTimerBar:SetCallback("OnEnd",DEDKP_BidWindow_BidTimerBar_OnEnd)
	DERR.BidWindow:AddChild(DERR.BidWindow.BidTimerBar)

	if next(DEDKPPoints[DERR_REALM][DERR_GUILD]) == nil then
		DEDKPPoints[DERR_REALM][DERR_GUILD] = DraconisErus_Guild_GetEmptyPoints()
	end

	DERR.BidWindow.BidTimerBar:SetMaxValue(timer)
	DERR.BidWindow.BidTimerBar:Start()

	DEDKP_BidWindow_SetItem(item_id)

	DraconisErus_SetWindowPosition("BidWindow", DERR.BidWindow)

end

function DEDKP_BidWindow_BidButton_OnClick()
	local bidAmount = DERR.BidWindow.BidAmount:GetText()
	bidAmount = tonumber(bidAmount)
	if bidAmount == nil then
		DraconisErusRaidRecorder:Print("|cFFFF0000Invalid bid amount!|r")
	elseif bidAmount < 0 then
		DraconisErusRaidRecorder:Print("|cFFFF0000Bid cannot be negative!|r")
	elseif bidAmount > points and points >= 0 then
		DraconisErusRaidRecorder:Print("|cFFFF0000You cannot bid more points than you have!|r")
	elseif bidAmount > points and points < 0 and bidAmount ~= 0 then
		DraconisErusRaidRecorder:Print("|cFFFF0000You can bid 0 points to show interest when you have negative points.|r")
	elseif bidAmount > 300 then
		DraconisErusRaidRecorder:Print("|cFFFF0000Bid exceeds the maximum of 300!|r")
	else
		DraconisErus_Comm_Send("DERRBID", bidAmount, "WHISPER", DERR_BID_OFFICER)
		DraconisErusRaidRecorder:Print("Bid submitted in the amount of "..bidAmount..".")
	end
end

function DEDKP_BidWindow_BidTimerBar_OnEnd()
	DERR.BidWindow.BidTimerBar:Stop()
	DEDKP_BidWindow_OnClose(DERR.BidWindow)
end

function DEDKP_BidWindow_ConfirmBid(bid)
	if DERR.BidWindow ~= nil then
		DERR.BidWindow:SetStatusText("Your bid of "..bid.." is confirmed.")
	end
end

function DEDKP_BidWindow_LootIcon_OnEnter(widget)
	GameTooltip:SetOwner(widget.frame)
	GameTooltip:SetPoint("TOPLEFT")
	GameTooltip:SetHyperlink(loot_link)
	GameTooltip:Show()
end

function DEDKP_BidWindow_LootIcon_OnLeave()
	GameTooltip:Hide()
end

function DEDKP_BidWindow_OnClose(widget)
	if DERR.BidWindow ~= nil then
		DERR.BidWindow.BidTimerBar:Stop()
		DraconisErus_SaveWindowPosition("BidWindow", widget)
		widget:ReleaseChildren()
		widget:Hide()
		DERR.BidWindow = nil
	end
	DraconisErus_Comm_Send("DERRBIDDONE", "", "WHISPER", DERR_BID_OFFICER)
end

AceGUI:RegisterLayout("DERR_Layout",
	function(content, children)
		local top = 0
		for i = 1, #children do
			local child = children[i]
			child.frame:SetPoint("TOP",content,"TOP",0,top)
			child.frame:Show()
			top = top - child.frame:GetHeight() - 5
		end
	end)