function DraconisErusRaidRecorder:BOSS_KILL(event, ...)
	local encounterID, encounterName = ...
	if not DraconisErus_PlayerAuth(0,1,1) then return end
	if encounterID == 610 then	--according to DBM, EE does not work for Razorgore, using BOSS_KILL
		if DERR_ENCOUNTER[encounterID] then
			local bossname = DERR_ENCOUNTER[encounterID]["Name"]
			DraconisErus_Debug("BOSS_KILL:"..bossname)
			DraconisErus_RaidEvent_BossKill(bossname, GetZoneText())
		end
	end
end

function DraconisErusRaidRecorder:ENCOUNTER_START(event, ...)
	local encounterID, encounterName, difficultyID, groupSize = ...
	DraconisErus_Debug("ENCOUNTER_START:"..encounterName)
end

function DraconisErusRaidRecorder:ENCOUNTER_END(event, ...)
	local encounterID, encounterName, difficultyID, groupSize, success = ...
	if not DraconisErus_PlayerAuth(0,1,1) then return end
	if encounterID ~= 610 then	--according to DBM, EE does not work for Razorgore, using BOSS_KILL
		if success and DERR_ENCOUNTER[encounterID] then
			local bossname = DERR_ENCOUNTER[encounterID]["Name"]
			DraconisErus_Debug("ENCOUNTER_END:"..bossname)
			DraconisErus_RaidEvent_BossKill(bossname, GetZoneText())
		end
	end
end

function DraconisErusRaidRecorder:CHAT_MSG_LOOT(event, msg)
	if not DraconisErus_PlayerAuth(0,1,1) then return end
	local player, itemLink, amount = DraconisErus_GetLoot(msg)
	if itemLink then
		local loot_id = tonumber(select(3,strfind(itemLink, "item:(%d+)")))
		if loot_id ~= nil then
			local item = Item:CreateFromItemID(loot_id)
			item:ContinueOnItemLoad(function()
				local itemName,itemLink,itemRarity,_,_,_,_,_,_,_,_,itemType, itemSubtype = GetItemInfo(loot_id)
				if itemRarity >= 4 or itemType == 9 or itemType == 7 or (itemType == 15 and itemSubtype == 1) then
					local data = {
						["Time"] = GetServerTime(),
						["ID"] = loot_id,
						["Name"] = itemName,
						["Player"] = player,
						["Amount"] = amount
					}
					if DEDKPLoot[DERR_REALM][DERR_GUILD][DEDKPSettings["RaidID"]] == nil then
						DEDKPLoot[DERR_REALM][DERR_GUILD][DEDKPSettings["RaidID"]] = {}
					end
					table.insert(DEDKPLoot[DERR_REALM][DERR_GUILD][DEDKPSettings["RaidID"]], data)
				end
			end)
		end
	end
end

function DraconisErusRaidRecorder:CHAT_MSG_SYSTEM(event, msg)
	local ERR_FRIEND_OFFLINE_S = string.gsub(_G.ERR_FRIEND_OFFLINE_S, "%%s", "(.+)");
	local ERR_RAID_MEMBER_ADDED_S = string.gsub(_G.ERR_RAID_MEMBER_ADDED_S, "%%s", "(.+)");
	local ERR_RAID_MEMBER_REMOVED_S = string.gsub(_G.ERR_RAID_MEMBER_REMOVED_S, "%%s", "(.+)");
	local player
	
	player = string.match(msg, ERR_FRIEND_OFFLINE_S)
	if player then
		player = DraconisErus_FixName(player)
		DEDKP_VersionWindow_RemoveVersion(player)
		if DERR.VersionWindow ~= nil then DEDKP_VersionWindow_LoadVersions() end
	end

	player = string.match(msg, ERR_RAID_MEMBER_ADDED_S)
	if player then
		if DEDKPSettings["RaidStarted"] then
			player = DraconisErus_FixName(player)
			if DraconisErus_Guild_HasPlayer(player) then
				if not DraconisErus_Queue_HasPlayer(player) then
					DraconisErus_RaidEvent_PlayerJoined(player)
				end
			end
			DraconisErus_Queue_Remove(player)
		else
			player = DraconisErus_FixName(player)
			DraconisErus_Queue_Remove(player)
		end
	end

	player = string.match(msg, ERR_RAID_MEMBER_REMOVED_S)
	if player then
		if DEDKPSettings["RaidStarted"] then
			player = DraconisErus_FixName(player)
			if DraconisErus_Guild_HasPlayer(player) then
				if not DraconisErus_Queue_HasPlayer(player) then
					DraconisErus_RaidEvent_PlayerLeft(player)
				end
			end
		end
	end
end

function DraconisErusRaidRecorder:GROUP_ROSTER_UPDATE()
	if not IsInGroup() then
		if DERR_INGROUP then
			DraconisErus_Debug("GROUP_ROSTER_UPDATE_GROUP_LEAVE")
		else
			DraconisErus_Debug("GROUP_ROSTER_UPDATE_RAID_LEAVE")
		end
		DEDKPSettings["RaidStarted"] = false
		DEDKPSettings["RaidQueue"] = {}
		DERR_RELOAD = true
		DERR_INGROUP = false
		draconiserus_inraid = false
	elseif IsInGroup() and not IsInRaid() then -- we are in a group
		DraconisErus_Debug("GROUP_ROSTER_UPDATE_GROUP")
		DERR_INGROUP = true
		draconiserus_inraid = false
	elseif IsInRaid() and DERR_INGROUP then -- we are in a raid that was just converted from a party
		DraconisErus_Debug("GROUP_ROSTER_UPDATE_RAID_CONVERT")
		DERR_INGROUP = false
		draconiserus_inraid = true
		DERR_RELOAD = false
	else
		DraconisErus_Debug("GROUP_ROSTER_UPDATE_RAID("..tostring(DERR_RELOAD)..")")
		--If our first GROUP_ROSTER_UPDATE after a login happens while we are in a raid,
		--we need to sync
		if DERR_RELOAD then
			DERR_RELOAD = false
			DraconisErus_Comm_Send("DERRPING","","RAID")
			C_Timer.After(0,function()
				C_Timer.After(5,function()
					DraconisErus_Raid_Sync()
				end)
			end)
		end
	end
end

function DraconisErusRaidRecorder:LOOT_OPENED()
	if not DraconisErus_PlayerAuth(1,1,1) then return end
	if not DERR_INBID then
		C_Timer.After(0,function()
			C_Timer.After(1,function()
				DraconisErus_LootWindowHook()
			end)
		end)
	end
end

function DraconisErusRaidRecorder:GUILD_ROSTER_UPDATE()
	if IsInGuild() then
		if DERR_LOADED == false then
			C_Timer.After(0,function()
				C_Timer.After(5,function()
					DraconisErus_Init()
				end)
			end)
			return
		else
			DraconisErus_Guild_Save()
		end
	else
		DraconisErus_Unload()
	end
end

function DraconisErusRaidRecorder:PLAYER_LEVEL_UP(event, level)
	DraconisErus_Comm_Send("DERRLEVEL",level,"GUILD")
	DraconisErus_Guild_Save(DERR_PLAYER,level)
end

function DraconisErusRaidRecorder:ZONE_CHANGED()
	DraconisErus_RaidEvent_ZoneChange()
end

function DraconisErusRaidRecorder:ZONE_CHANGED_INDOORS()
	DraconisErus_RaidEvent_ZoneChange()
end

function DraconisErusRaidRecorder:ZONE_CHANGED_NEW_AREA()
	DraconisErus_RaidEvent_ZoneChange()
end

function DraconisErus_LootWindowHook()			-- hook function into LootFrame window (BREAKS if more than 4 loot slots... trying to fix)
	if not DEDKPSettings["RaidStarted"] then return end
	local num = GetNumLootItems()
	if getglobal("ElvLootSlot1") then 			-- fixes hook for ElvUI loot frame
		for i = 1, num do
			DraconisErus_Debug("ElvLootSlot"..i.."/"..num)
			getglobal("ElvLootSlot"..i):HookScript("OnClick", function(self, button)
			if (button == "RightButton" and IsControlKeyDown() and IsAltKeyDown()) then
	        		itemLink = GetLootSlotLink(i)
					if itemLink ~= nil then
						if UnitIsGroupAssistant("player") or UnitIsGroupLeader("player") then
							DEDKP_BidManager_SetItem(itemLink)
						else
							DraconisErusRaidRecorder:Print("|cFFFF0000You must have raid assist to handle bidding.|r")
						end
					end
		        end
			end)
		end
	else
		for i = 1, num do 
			getglobal("LootButton"..i):HookScript("OnClick", function(self, button)
			if (button == "RightButton" and IsControlKeyDown() and IsAltKeyDown()) then
	        		itemLink = GetLootSlotLink(i)
					if itemLink ~= nil then
						if UnitIsGroupAssistant("player") or UnitIsGroupLeader("player") then
							DEDKP_BidManager_SetItem(itemLink)
						else
							DraconisErusRaidRecorder:Print("|cFFFF0000You must have raid assist to handle bidding.|r")
						end
					end
		        end
			end)
		end
	end
	if UnitIsGroupAssistant("player") or UnitIsGroupLeader("player") then
		DraconisErusRaidRecorder:Print("Loot Ready.")
	end
end

